@@ -46,7 +46,7 @@ class Video(Scene):
                  mindim=512,
                  dt=1,
                  fetch=True,
-                 attributes=None
+                 attributes=None,
     ):
         assert (mp4file is not None or mp4url is not None), "Invalid input - Must provide either mp4file or mp4url"
         assert (jsonurl is not None or jsonfile is not None), "Invalid input - Must provide either jsonurl or jsonfile"
@@ -55,13 +55,11 @@ class Video(Scene):
 
         # AWS credentials (if needed) must be set by pycollector.user
         if ((jsonurl is not None and (jsonfile is None or not os.path.exists(jsonfile))) or
-            (mp4url is not None and (mp4file is None or not os.path.exists(mp4file)))):            
+            (mp4url is not None and (mp4file is None or not os.path.exists(mp4file)))):
              assert 'VIPY_AWS_ACCESS_KEY_ID' in os.environ and 'VIPY_AWS_SECRET_ACCESS_KEY' in os.environ, "AWS access keys not found - Log in using pycollector.user"
         
         # Vipy video constructor
-        mp4url = mp4url.replace('+',' ') if mp4url is not None else mp4url  # for cut and paste from AWS console
-        jsonurl = jsonurl.replace('+',' ') if jsonurl is not None else jsonurl  # for cut and paste from AWS console        
-        super(Video, self).__init__(url=mp4url, filename=mp4file, attributes=attributes)
+        super().__init__(url=mp4url, filename=mp4file, attributes=attributes)
 
         # Video attributes
         #self._quicklook_url = "https://%s.s3.amazonaws.com/Quicklooks/%%s_quicklook_%%s_%%d.jpg" % (backend().s3_bucket())   # FIXME: pycollector.admin.video
@@ -81,7 +79,7 @@ class Video(Scene):
                                                                                   if self.timestamp() is not None
                                                                                   else str(None),
                                                                                   self.activity_categories(),
-                                                                                  str(super(Video, self).__repr__())))
+                                                                                  str(super().__repr__())))
             
     def _load_json(self):
         """Lazy JSON download, parse, and import"""
@@ -158,9 +156,26 @@ class Video(Scene):
             print('[pycollector.video]: empty JSON "%s" - SKIPPING' % jsonfile)
             d = None
 
-            
+        # Backwards compatible video import: should not be necessary with new app release
+        if not 'category' in d['metadata']:
+            vipy.util.try_import('pycollector.admin.globals', message="Not authorized - Old style JSON requires admin access")
+            from pycollector.admin.globals import backend  
+            if not backend().collections().iscollectionid(d["metadata"]["collection_id"]):
+                print('[pycollector.video]: invalid collection ID "%s" - SKIPPING' % d["metadata"]["collection_id"])
+                d = None
+            else:
+                # Fetch labels from backend (yuck)
+                d['metadata']['collection_name'] = backend().collection().id_to_name(d["metadata"]["collection_id"])
+                d['metadata']['category'] = ','.join([backend().collection()[d["metadata"]["collection_id"]].shortname_to_activity(a["label"]) for a in d['activity']])
+                d['metadata']['shortname'] = ','.join([a["label"] for a in d['activity']])
+                
+        else:
+            # New style JSON: use labels stored directly in JSON
+            pass
+        
         # Import JSON into scene
         if d is not None:
+
             # TODO - Replace with video_data
             collection_name = d['metadata']['collection_name']
             
@@ -214,8 +229,22 @@ class Video(Scene):
                     self.add(t)
                 d_trackid_to_track[t.id()] = t
 
+            
+            # Category variants:  a_category_name#Variant1=A&Joint=a_joint_label:Short Label&Variant2=B
+            variant = {}
+            d_shortname_to_category = {s:c for (s,c) in zip(d['metadata']['shortname'].split(','), d['metadata']['category'].split(','))}            
+            if '#' in d['metadata']['category']:
+                d_shortname_to_category = {s:c.split('#')[0] for (s,c) in d_shortname_to_category.items()}
+                variantlist = list(set([c.split('#')[1] if '#' in c else None for c in d['metadata']['category'].split(',')]))
+                if len(variantlist) != 1:
+                    print('[pycollector.video]: WARNING - Ignoring mixed variant "%s"' % str(variantlist))
+                elif any(['=' not in v or v.count('&') != (v.count('=')-1) for v in variantlist]):
+                    print('[pycollector.video]: WARNING - Ignoring invalid variant "%s"' % str(variantlist))                    
+                else:
+                    variant = {k.split('=')[0]:k.split('=')[1] for k in variantlist[0].split('&')}
+            self.attributes['variant'] = variant
+
             # Import activities
-            d_shortname_to_category = {s:c for (s,c) in zip(d['metadata']['shortname'].split(','), d['metadata']['category'].split(','))}
             for a in d["activity"]:
                 try:
                     if (d["metadata"]["collection_id"] == "P004C009" and d["metadata"]["device_identifier"] == "android"):
@@ -230,7 +259,6 @@ class Video(Scene):
                         shortlabel = "Buying (Cashier)"
                     else:
                         shortlabel = a["label"]
-                    shortlabel = shortlabel.lower()
 
                     #category = backend().collection()[d["metadata"]["collection_id"]].shortname_to_activity(a["label"])
                     category = d_shortname_to_category[a['label']]
@@ -241,15 +269,17 @@ class Video(Scene):
                                                     tracks=d_trackid_to_track,
                                                     framerate=d["metadata"]["frame_rate"],
                                                     attributes=d["metadata"]))
-                    
-                    if not vipy.version.is_at_least("0.8.3"):
-                        if int(a["start_frame"]) >= int(a["end_frame"]):
-                            raise ValueError(
-                                "degenerate start/end frames %d/%d",
-                                int(a["start_frame"]),
-                                int(a["end_frame"]),
-                            )
 
+                    # Joint activity?
+                    if 'Joint' in variant:
+                        self.add(vipy.activity.Activity(category=variant['Joint'].split(':')[0],
+                                                        shortlabel=variant['Joint'].split(':')[1] if ':' in variant['Joint'] else None,
+                                                        startframe=int(a["start_frame"]),
+                                                        endframe=int(a["end_frame"]),
+                                                        tracks=d_trackid_to_track,
+                                                        framerate=d["metadata"]["frame_rate"],
+                                                        attributes=d["metadata"]))
+                                            
                 except Exception as e:
                     print(
                         '[pycollector.video]: Filtering invalid activity "%s" with error "%s" for videoid=%s'
@@ -280,11 +310,15 @@ class Video(Scene):
             self._is_json_loaded = False
 
         # Resample tracks
-        if self._dt > 1:
+        if self._dt > 1 and self._is_json_loaded:
             self.trackmap(lambda t: t.resample(self._dt))
         
         return self
 
+    def variant(self):
+        """Category variant"""
+        return self.attributes['variant'] if 'variant' in self.attributes else None
+    
     def geolocation(self):
         assert 'ipAddress' in self.metadata(), "Invalid JSON"
         url = 'http://api.geoiplookup.net/?query=%s' % self.metadata()['ipAddress']
@@ -299,7 +333,7 @@ class Video(Scene):
         return self.fetchvideo()
 
     def fetchvideo(self, ignoreErrors=False):
-        super(Video, self).fetch()
+        super().fetch()
         return self
 
     def fetchjson(self):
@@ -395,6 +429,11 @@ class Video(Scene):
     def collectionid(self):
         return self.attributes["collection_id"] if "collection_id" in self._load_json().attributes else None
 
+    def collection_name(self):
+        return self.attributes["collection_name"] if "collection_name" in self._load_json().attributes else None
+    def collection(self):
+        return self.collection_name()
+    
     def duration(self):
         """Video length in seconds"""
         return float(self.attributes["duration"]) if "duration" in self._load_json().attributes else 0.0
@@ -430,4 +469,5 @@ def search():
 
 def last(n=1):
     import pycollector.project    
-    return pycollector.project.Project(since='2020-09-01', last=n).last(n)    
+    return pycollector.project.Project(since='2020-09-01', last=n).last(n)
+
